"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var util = require("util");
var _ = require("lodash");
var i18next = require("i18next");
var Backend = require("i18next-node-fs-backend");
var VirtualModulePlugin = require('virtual-module-webpack-plugin');
var readFile = util.promisify(fs.readFile);
function extractArgs(arg, warning) {
    switch (arg.type) {
        case 'Literal':
            return arg.value;
        case 'Identifier':
            return arg.name;
        case 'ObjectExpression':
            var res = {};
            for (var i in arg.properties) {
                res[extractArgs(arg.properties[i].key)] = extractArgs(arg.properties[i].value);
            }
            return res;
        default:
            if (warning) {
                warning("unable to parse arg " + arg);
            }
            return null;
    }
}
function getPath(template, language, namespace) {
    if (language !== undefined) {
        template = template.replace("{{lng}}", language);
    }
    if (namespace !== undefined) {
        template = template.replace("{{ns}}", namespace);
    }
    return template;
}
var I18nextPlugin = (function () {
    function I18nextPlugin(option) {
        this.option = _.defaults(option, {
            functionName: "__",
            defaultNamespace: "translation",
            namespaces: [option.defaultNamespace || "translation"],
            outPath: option.resourcePath
        });
        i18next.use(Backend);
    }
    I18nextPlugin.prototype.apply = function (compiler) {
        var _this = this;
        // provide config via virtual module plugin
        compiler.apply(new VirtualModulePlugin({
            moduleName: path.join(__dirname, "config.js"),
            contents: "exports = module.exports = {\n                __esModule: true,\n                RESOURCE_PATH: \"" + this.option.outPath + "\",\n                LANGUAGES: " + JSON.stringify(this.option.languages) + ",\n                DEFAULT_NAMESPACE: \"" + this.option.defaultNamespace + "\"\n};"
        }));
        i18next.init({
            preload: this.option.languages,
            ns: this.option.namespaces,
            fallbackLng: false,
            defaultNS: this.option.defaultNamespace,
            saveMissing: true,
            missingKeyHandler: this.onKeyMissing.bind(this),
            backend: {
                loadPath: this.option.resourcePath
            }
        });
        this.context = compiler.options.context || "";
        compiler.plugin("compilation", function (compilation, data) {
            // reset for new compliation
            _this.missingKeys = {};
            i18next.reloadResources(_this.option.languages);
            _this.compilation = compilation;
            data.normalModuleFactory.plugin("parser", function (parser) {
                parser.plugin("call " + _this.option.functionName, _this.onTranslateFunctionCall.bind(_this));
            });
        });
        compiler.plugin("emit", this.onEmit.bind(this));
        compiler.plugin("after-emit", this.onAfterEmit.bind(this));
    };
    I18nextPlugin.prototype.onEmit = function (compilation, callback) {
        // emit translation files
        var promises = [];
        for (var _i = 0, _a = this.option.languages; _i < _a.length; _i++) {
            var lng = _a[_i];
            var resourceTemplate = path.join(this.context, getPath(this.option.resourcePath, lng));
            try {
                var resourceDir = path.dirname(resourceTemplate);
                fs.statSync(resourceDir);
                // compilation.contextDependencies.push(resourceDir);
            }
            catch (e) {
            }
            var _loop_1 = function (ns) {
                var resourcePath = getPath(resourceTemplate, undefined, ns);
                var outPath = getPath(this_1.option.outPath, lng, ns);
                promises.push(readFile(resourcePath).then(function (v) {
                    compilation.assets[outPath] = {
                        size: function () { return v.length; },
                        source: function () { return v; }
                    };
                    compilation.fileDependencies.push(path.resolve(resourcePath));
                }).catch(function () {
                    compilation.warnings.push("Can't emit " + outPath + ". It looks like " + resourcePath + " is not exists.");
                }));
            };
            var this_1 = this;
            for (var _b = 0, _c = this.option.namespaces; _b < _c.length; _b++) {
                var ns = _c[_b];
                _loop_1(ns);
            }
        }
        Promise.all(promises).then(function () { return callback(); }).catch(callback);
    };
    I18nextPlugin.prototype.onAfterEmit = function (compilation, callback) {
        var _this = this;
        // write missing 
        Promise.all(_.map(this.missingKeys, function (namespaces, lng) { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, _.map(namespaces, function (keys, ns) { return __awaiter(_this, void 0, void 0, function () {
                        var _this = this;
                        return __generator(this, function (_a) {
                            return [2 /*return*/, new Promise(function (resolve) {
                                    var missingPath = path.join(_this.context, getPath(_this.option.pathToSaveMissing, lng, ns));
                                    var stream = fs.createWriteStream(missingPath, {
                                        defaultEncoding: "utf-8"
                                    });
                                    keys = _.sortedUniq(_.sortBy(keys));
                                    console.log(keys);
                                    stream.write("{\n");
                                    stream.write(_.map(keys, function (key) { return "\t\"" + key + "\": \"" + key + "\""; }).join(",\n"));
                                    stream.write("\n}");
                                    stream.on("close", function () { return resolve(); });
                                    compilation.warnings.push("missing translation " + keys.length + " keys in " + lng + "/" + ns);
                                })];
                        });
                    }); })];
            });
        }); })).then(function () { return callback(); }).catch(callback);
    };
    I18nextPlugin.prototype.onTranslateFunctionCall = function (expr) {
        var _this = this;
        var args = expr.arguments.map(function (arg) { return extractArgs(arg, _this.warningOnCompilation.bind(_this)); });
        for (var _i = 0, _a = this.option.languages; _i < _a.length; _i++) {
            var lng = _a[_i];
            var keyOrKeys = args[0];
            var option = Object.assign(_.defaults(args[1], {}), {
                lng: lng,
                defaultValue: null
            });
            i18next.t(keyOrKeys, option);
        }
    };
    I18nextPlugin.prototype.onKeyMissing = function (lng, ns, key, __) {
        var p = [lng, ns];
        var arr = _.get(this.missingKeys, p);
        if (arr === undefined) {
            _.set(this.missingKeys, p, []);
            arr = _.get(this.missingKeys, p);
        }
        arr.push(key);
    };
    I18nextPlugin.prototype.warningOnCompilation = function (msg) {
        if (this.compilation) {
            this.compilation.warnings.push(msg);
        }
    };
    return I18nextPlugin;
}());
exports.default = I18nextPlugin;
//# sourceMappingURL=index.js.map